﻿//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.


#include "EspfRequestTool.h"
#include "IpRequestSender.h"
#include "PipeRequestSender.h"

/****************************//**
*
* @brief   entry point for the program
* @author  Evolis R&D Software
* @param   standard command line arguments
* @return
*  - @c MAIN_ERR_OPT       (1)  if an error occurs during the command line parsing
*  - @c MAIN_ERR_CRITICAL  (2)  if an error occurs during the communication with the printer
*  - @c MAIN_OK            (0)  else
*
*/

int main(int argc, char* argv[])
{
	char inputFile[STR_MAX_SZ] = {0};		///< input filename
	char serverAddress[STR_MAX_SZ] = {0};	///< server adress
	char ip[STR_MAX_SZ] = {0};				///< ip
	char port[STR_MAX_SZ] = {0};			///< port
	char outputFile[STR_MAX_SZ] = {0};		///< output filename
	char pipename[STR_MAX_SZ] = { 0 };		///< pipe name
	RequestType requestType = UnknownRequestType;
	const char splitchar = ':';
	int result = 1;
	unsigned long maxAnswerSize = STR_DEFAULTANSWER_SZ;

	if (argc < 5)
	{
		printf("WARNING: invalid argument length\n\n");
		printUsage("EspfRequestTool");
		return MAIN_ERR_OPT;
	}

	for (int i = 1; i < argc; i++)
	{
		// commandline parameter should begin with '-' 
		if (argv[i][0] == '-')
		{
			switch (argv[i][1])
			{
				case 'i': case 'I': // DEFINE INPUT FILENAME
				{
					if (i < argc - 1)
					{
						i++;
						strncpy_s(inputFile, STR_MAX_SZ, argv[i], STR_MAX_SZ);
					}
					else
					{
						printf("WARNING: missing argment (-input filename)\n");
						return MAIN_ERR_OPT;
					}
					break;
				}
				case 's': case 'S': // SERVER ADDRESS 
				{
					if (i < argc - 1)
					{
						i++;
						strncpy_s(serverAddress, STR_MAX_SZ, argv[i], STR_MAX_SZ);
						requestType = IpRequestType;
					}
					else
					{
						printf("WARNING: missing argment (-s address)\n");
						return MAIN_ERR_OPT;
					}
					break;
				}
				case 'p': case 'P': // SERVER ADDRESS 
				{
					if (i < argc - 1)
					{
						i++;
						strncpy_s(pipename, STR_MAX_SZ, argv[i], STR_MAX_SZ);
						requestType = PipeRequestType;
					}
					else
					{
						printf("WARNING: missing argment (-p pipe)\n");
						return MAIN_ERR_OPT;
					}
					break;
				}
				case 'a': case 'A': // ANSWER BUFFER SIZE 
				{
					if (i < argc - 1)
					{
						i++;						
						maxAnswerSize = strtoul(argv[i], NULL, 0);
						if (maxAnswerSize == 0)
						{
							printf("WARNING: incorrect argment value (-a size)\n");
							return MAIN_ERR_OPT;
						}
					}
					else
					{
						printf("WARNING: missing argment (-a size)\n");
						return MAIN_ERR_OPT;
					}
					break;
				}
				case 'o': case 'O': // OUTPUT FILENAME
				{
					if (i < argc - 1)
					{
						i++;
						strncpy_s(outputFile, STR_MAX_SZ, argv[i], STR_MAX_SZ);
					}
					else
					{
						printf("WARNING: missing argment (-o filename)\n");
						return MAIN_ERR_OPT;
					}
					break;
				}
				default:
					printUsage(argv[0]);
			}
		}
		else
		{
			printf("Unknown argument type\n");
			return MAIN_ERR_OPT;
		}
	}
	
	if (requestType == IpRequestType)
	{
		for (size_t i = strlen(serverAddress); i > 0; i--)
		{
			if (serverAddress[i] == splitchar)
			{
				for (size_t j = 0; j < (strlen(serverAddress) - i); j++)
				{
					port[j] = serverAddress[i + j + 1];
				}

				for (size_t k = 0; k < i; k++)
				{
					ip[k] = serverAddress[k];
				}
			}
		}
		
		return sendRequestIp(inputFile, ip, port, outputFile, maxAnswerSize);

	}
	else if (requestType == PipeRequestType)
	{
		return sendRequestPipe(inputFile, pipename, NULL, outputFile, maxAnswerSize);
	}
	else
	{ 
		fprintf(stderr, "at least one parameter : '-s ip' or '-p pipe' should be used\n");
		return MAIN_ERR_OPT;
	}
}

//-------------------------------------------------------------------

/**
* @brief print usage to console
*
* @param exeName the name of the executable (usually argv[0])
*/
void printUsage(char* exeName)
{
	printf("Evolis ESPF Request Tool - ");
	printf("Usage: %s -i file1 {-s ip|-p pipe} [-a size] -o file2\n", exeName);
	printf("\n -i  : input filename");
	printf("\n -s  : server address (eg 127.0.0.1:18000)");
	printf("\n -p  : pipe name (eg \\\\.\\pipe\\EspfServer00)");
	printf("\n -a  : size in bytes to replace default answer buffer size (4096 bytes)");
	printf("\n -o  : output filename\n");
	printf("\n Example :  EspfRequestTool.exe -i test.txt -s 11.1.24.210:18000 -o output.txt\n");
}

//eof