/*
     This file is part of the Evolis EVOSDK1.

     The Evolis EVOSDK1 is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation, either version 3 of the License, or
     any later version.

     The Evolis EVOSDK1 is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.

     You should have received a copy of the GNU Lesser General Public License
     along with the Evolis EVOSDK1.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "demo.h"

#include "EvoSdk1Lib1.h"

#include <functional>
#include <iostream>

using namespace std;

typedef std::function<void(Data* data)> callback_t;

struct MenuItem
{
    MenuItem(const char* label, const callback_t& func)
        : label(label), func(func) {}
    const char* label;
    callback_t func;
};

static void choosePrinter(Data* data)
{
    HANDLE handle;

    printf("=== TARGET PRINTER ===\n");
    handle = EPSDK_GAPI_CreateCommunicationSettings(EPSDK_CT_Pipe, ".", data->serverPipe);
    if (handle != NULL) {
        char*    reply;
        DWORD    ec;

        ec = EPSDK_Supervision_EnumEvolisSupervisedPrinters(handle, 0, data->modelName, &reply);
        // If more than one printer available then "reply" will contain
        // a string like "<PRINTER-0>;<PRINTER-1>;<PRINTER-N>" with
        // <PRINTER-X> replaced with the name of the printer.

        if (ec == EPSDK_NO_ERROR) {
            auto    tokens = split(reply, ';');
            size_t  choice = tokens.size();

            if (tokens.empty()) {
                printf(" >> No printer available for this model.\n");
                data->printerAvailable = false;
            } else {
                if (tokens.size() == 1) {
                    choice = 0;
                    printf(" >> Only one printer available.\n");
                    printf(" >> Selecting printer '%s'\n", tokens[0].data());
                }
                else {
                    for (size_t i = 0; i < tokens.size(); ++i)
                        printf("    %d - %s\n", i, tokens[i].data());
                    while (choice >= tokens.size()) {
                        printf(" ?? Please choose a printer number : ");
                        if (!(std::cin >> choice)) {
                            std::cin.clear();
                            std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
                        }
                    }
                }
                strcpy(data->printerName, tokens[choice].data());
                data->printerAvailable = true;
            }
        }
        EPSDK_GAPI_DestroyCommunicationSettings(handle);
    }
}

static void inputModelName(Data* data)
{
    char modelName[64];

    printf("=== TARGET PRINTER MODEL ===\n");
    printf(" ?? Please input your printer model name : ");

    scanf(" %[^\n]s", modelName);

    strcpy(data->modelName, modelName);
    strcpy(data->printerName, "");
    choosePrinter(data);
}

static void checkPrinterStatus(const char* printerName, const char* serverPipe, char ** majorStatus, char ** minorStatus)
{
    DWORD errorCode;
    HANDLE comHandle;

    comHandle = EPSDK_GAPI_CreateCommunicationSettings(EPSDK_CT_Pipe, ".", serverPipe);

    HANDLE printerHandle = EPSDK_GAPI_OpenPrinter(printerName, comHandle);

    if (printerHandle == NULL) {
        printf("    Error: Error getting the printer handle.\n");
    }
    else {
        errorCode = EPSDK_Supervision_GetDeviceState(printerHandle, majorStatus, minorStatus);
        if (errorCode != EPSDK_NO_ERROR) {
            printf("    Error: EPSDK_Supervision_GetDeviceState() returned with code '%d'.\n", errorCode);
        }
        EPSDK_GAPI_ClosePrinter(printerHandle);
        EPSDK_GAPI_DestroyCommunicationSettings(comHandle);
    }
}

static void quit(Data* data)
{
    data->quit = true;
}

static void displayHelp() {
    printf("----------------------------\n");
    printf("List of parameters :\n");
    printf("----------------------------\n");
    printf("USAGE : -h  | -?  | --help        : Show this help section\n");
    printf("USAGE : -p        | --printertype : To select a printer type different than the default one(\"Evolis Primacy 2\").\n");
    printf("USAGE : -spi      | --serverpipe  : For printers related to the Evolis Premium Suite 2 (Primacy 2) do not specify any argument. It will use the default value: Espf2Server00. For Printers working with EPS 1 you must adapt the value and use the following one: ESPFServer00.\n");
}

static char* getParameterValue(char* args[], int index, char* parameter) {

    if (sizeof(args) >= (index + 1) && strncmp("-", args[index + 1], 1) != 0) {
        return args[index + 1];
    }

    std::cout << "Error missing parameter value for parameter: " + std::string(parameter) + "\n";

    return 0;
}

/**
 * Main for demo EvoSdk1Lib1
 * The binary takes the model as argument, defaults to "Evolis Primacy 2".
 */
int main(int argc, char* argv[])
{
    const char* printerName = "Evolis Primacy 2";
    const char* serverPipe = "Espf2Server00";

    int cpt = 0;

    for (int i = 0; i < argc; i++) {
        if (argv[i][0] == '-') {
            if (strncmp("-h", argv[i], 2) == 0 || strncmp("-?", argv[i], 2) == 0 || strncmp("-help", argv[i], 5) == 0) {
                displayHelp();
                return 0;
            }

            if (strncmp("-p", argv[i], 2) == 0 || strncmp("--printertype", argv[i], 13) == 0) {
                printerName = getParameterValue(argv, cpt, argv[i]);

                if (printerName == 0) {
                    return 0;
                }
            }

            if (strncmp("-spi", argv[i], 4) == 0 || strncmp("--serverpipe", argv[i], 12) == 0) {
                serverPipe = getParameterValue(argv, cpt, argv[i]);

                if (serverPipe == 0) {
                    return 0;
                }
            }
        }

        cpt++;
    }

    Data data(serverPipe, printerName);

    std::vector<MenuItem> commandsPrinterAvailable = {
        {"Change the target printer model", inputModelName},
        {"Change the target printer", choosePrinter},
        {"Infos / Get printer infos", demoPrinterInfos},
        {"Infos / Get ribbon infos", demoRibbonInfos},
        {"Mag encoding demo", demoMagEncoding},
        {"Print demo / Simplex YMCKO", demoPrintSimplex},
        {"Print demo / Duplex YMCKOK", demoPrintDuplex},
        {"Supervision demo", demoSupervision},
        {"Support tools demo", demoSupportTools},
        {"Support tools / Print a test card", demoPrintTestCard},
        {"Support tools / Run cleaning cycle", demoCleaningCycle},
        {"Support tools / Send a custom command", demoSendCommand},
        {"Quit", quit}
    };
    std::vector<MenuItem> commandsPrinterUnavailable = {
        {"Change the target printer model", inputModelName},
        {"Change the target printer", choosePrinter},
        {"Quit", quit}
    };

    choosePrinter(&data);

    while (!data.quit) {
        char*  majorStatus;
        char*  minorStatus;

        std::vector<MenuItem> commands = data.printerAvailable ? commandsPrinterAvailable : commandsPrinterUnavailable;
        size_t choice = commands.size();

        checkPrinterStatus(data.printerName, serverPipe, &majorStatus, &minorStatus);

        printf("\n\n=== MENU ===\n\n");
        printf(" >> Server pipe          : %s\n", serverPipe);
        printf(" >> Printer model        : %s\n", data.modelName);
        printf(" >> Printer name         : %s\n", data.printerName);
        printf(" >> Printer major status : %s\n", majorStatus);
        printf(" >> Printer minor status : %s\n\n", minorStatus);

        for (size_t i = 0; i < commands.size(); ++i) {
            printf("    %2d - %s\n", i, commands[i].label);
        }
        printf("\n");

        while (choice >= commands.size()) {
            std::cout << " ?? Please, what is your option ? \n";
            if (!(std::cin >> choice)) {
                std::cin.clear();
                std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
            }
        }

        printf("\n\n");
        commands[choice].func(&data);
    }
    return 0;
}
