﻿//
// File    : CardDesigner.cs
// Author  : Evolis Card Printer
// History :
// 
//    19th feb 2013 - file creation
//

using System;
namespace SupervisorLayer
{
    /// <summary>
    /// IPrinterStatusHelper's implementation for Evolis Printer 
    /// </summary>
    internal class EvolisPrinterStatusHelper : IPrinterStatusHelper
    {
        #region Private Properties

        private const int StatusSize = 16;

        private string _printerAnswer;
        private byte[] _statusBits;

        #endregion

        #region Public Methods

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Constructor
        /// </summary>
        public EvolisPrinterStatusHelper()
        {
            _statusBits = new byte[StatusSize];
            _printerAnswer = "";
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Destructor
        /// </summary>
        ~EvolisPrinterStatusHelper()
        {
            _statusBits = null;
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Refresh the status structure.
        /// </summary>
        /// <param name="answer">The answer.</param>
        /// <returns>true if it is Ok; otherwise false</returns>
        public IPrinterStatusHelperError Update(byte[] answer)
        {
            if ((answer == null) || (answer.Length == 0))
            {
                return IPrinterStatusHelperError.NullData;
            }

            int position = 0;

            // look for first 0 byte
            _printerAnswer = GetPrinterAnswerEx(answer, ref position);

            position += 3; // jump information bytes related to printer's answer size  

            if (position >= answer.Length)
            {
                // no status
                return IPrinterStatusHelperError.EmptyStatus;
            }

            if ((answer[position] != StatusSize) || (position + StatusSize >= answer.Length))
            {
                // invalid status size
                return IPrinterStatusHelperError.DecodingError;
            }

            position++;

            try
            {
                Buffer.BlockCopy(answer, position, _statusBits, 0, StatusSize);
            }
            catch (Exception e)
            {
                // EVOLIS: Please log the message if necessary 
                return IPrinterStatusHelperError.MemoryError;
            }

            return IPrinterStatusHelperError.NoError;
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Gets the printer answer.
        /// </summary>
        /// <param name="answer">The answer.</param>
        /// <returns></returns>
        public static string GetPrinterAnswer(byte[] answer)
        {
            int position = 0;
            return GetPrinterAnswerEx(answer, ref position);
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public string GetPrinterAnswer()
        {
            return _printerAnswer;
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Determines whether [is printer claimed].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is printer claimed]; otherwise, <c>false</c>.
        /// </returns>
        /// <exception>InternalErrorException</exception>
        public bool IsPrinterClaimed()
        {
            return ((_statusBits[4] & 0x80) != 0);
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Determines whether [is printer ready to print].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is printer ready to print]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsPrinterBusy()
        {
            return ((_statusBits[7] & 0x01) != 0);
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Determines whether [is chip station available].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is chip station available]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsChipStationAvailable()
        {
            return ((_statusBits[1] & 0x08) != 0);
        }

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Determines whether [is card available].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is card available]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsCardAvailable()
        {
            return ((_statusBits[8] & 0x01) == 0);
        }

        /// <summary>
        /// Determines whether [is cover opened].
        /// </summary>
        /// <returns></returns>
        public bool IsCoverOpened()
        {
            return (((_statusBits[9] & 0x02) != 0) /* Default cover open */
                || ((_statusBits[12] & 0x02) != 0)); /* Error cover open */
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsRibbonDetected()
        {
            return ((_statusBits[9] & 0x01) == 0);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsRibbonWellDefined()
        {
            return (((_statusBits[5] & 0x10) == 0) &&   /* Ribbon known */
                ((_statusBits[6] & 0x04) == 0));        /* Right Ribbon Zone */
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsEndOfRibbon()
        {
            return ((_statusBits[9] & 0x08) != 0);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsErrorPrintHead()
        {
            return ((_statusBits[12] & 0x20) != 0);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsErrorCover()
        {
            return ((_statusBits[12] & 0x02) != 0);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsErrorRibbon()
        {
            return ((_statusBits[12] & 0x04) != 0);
        }

        ///<summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsErrorFeeder()
        {
            return ((_statusBits[12] & 0x08) != 0);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsErrorMechanical()
        {
            return ((_statusBits[12] & 0x01) != 0);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsCardInPrinter()
        {
            if (((_statusBits[4] & 0x10) == 0) && /* card not detected inside the flip station */
                ((_statusBits[4] & 0x08) == 0) && /* card not detected inside the ctless station */
                ((_statusBits[4] & 0x04) == 0) && /* card not detected inside the smart station */
                ((_statusBits[4] & 0x02) == 0))   /* card not detected inside the smart station */
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// Infoes this instance.
        /// </summary>
        /// <returns></returns>
        public string Info()
        {
            //return "Last Printer Answer:" + GetPrinterAnswer() + "\r\n" +
              return "IsPrinterClaimed: " + IsPrinterClaimed() + "\r\n" +
                "IsPrinterBusy: " + IsPrinterBusy() + "\r\n" +
                "IsChipStationAvailable: " + IsChipStationAvailable() + "\r\n" +
                "IsCardInPrinter: " + IsCardInPrinter() + "\r\n" +
                "IsCardAvailable: " + IsCardAvailable() + "\r\n" +
                "IsCoverOpened: " + IsCoverOpened() + "\r\n" +
                "IsRibbonDetected: " + IsRibbonDetected() + "\r\n" +
                "IsRibbonWellDefined: " + IsRibbonWellDefined() + "\r\n" +
                "IsEndOfRibbon: " + IsEndOfRibbon() + "\r\n" +
                "IsErrorPrintHead: " + IsErrorPrintHead() + "\r\n" +
                "IsErrorCover: " + IsErrorCover() + "\r\n" +
                "IsErrorFeeder: " + IsErrorFeeder() + "\r\n" +
                "IsErrorRibbon: " + IsErrorRibbon() + "\r\n" +
                "IsErrorMechanical: " + IsErrorMechanical() + "\r\n";
        }
        #endregion

        #region Private Methods

        // ------------------------------------------------------------------------------------------

        /// <summary>
        /// Gets the printer answer.
        /// </summary>
        /// <param name="answer">The answer.</param>
        /// <param name="position">The position.</param>
        /// <returns></returns>
        private static string GetPrinterAnswerEx(byte[] answer, ref int position)
        {
            while ((position < answer.Length) && (answer[position] != 0))
            {
                position++;
            }

            try
            {
                byte[] printerAnswerBytes = new byte[position];

                Buffer.BlockCopy(answer, 0, printerAnswerBytes, 0, position);
                System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
                return encoding.GetString(printerAnswerBytes);
            }
            catch (Exception e)
            {
                // EVOLIS: Please log the message if necessary 

            }

            return "";
        }

        #endregion
    }
}
