﻿/////////////////////////
//IMPORTANT INFORMATION
/////////////////////////
//
// Please, check if the print center is desactivated to avoid communication conflicts with the Evolis printer. 
//

using System;
using System.Runtime.InteropServices;
using SupervisorLayer;

namespace PrinterStatus
{
    class Program
    {
        #region Private Properties

        // ------------------------------------------------------------------------------------------
        private static IPrinterStatusHelper _statusHelper;
        private static byte[] _status;
        private const int BIGSTRSIZE = 4096;
        private static int _printerPtr;
        private static string _printername = "Evolis Primacy"; //Change the _printername with your printer name

        #endregion

        # region Native Low Communication Library

        /// <summary>
        /// open a connexion with the printer's port
        /// </summary>
        /// <param name="printerName">the printer name (according to the spooler)</param>
        /// <returns>a handle and NULL if the connexion failed</returns>
        [DllImport("iomem.dll", EntryPoint = "OpenPebble")]
        public static extern int OpenPebble([InAttribute]string printerName);

        /// <summary>
        /// close the connexion with the printer's port
        /// </summary>
        /// <param name="printerHandle">the handle provided by OpenPebble</param>
        /// <returns>the status of the operation</returns>
        [DllImport("iomem.dll", EntryPoint = "ClosePebble")]
        public static extern bool ClosePebble([InAttribute]int printerHandle);

        /// <summary>
        /// Read data from the printer
        /// </summary>
        /// <param name="printerHandle">the handle provided by OpenPebble</param>
        /// <param name="answer">address to an array</param>
        /// <param name="answerSize">the size of the answer's array (minimum 512 bytes)</param>
        /// <param name="dataSize">the number of char. need send by the printer</param>
        /// <returns>the status of the operation</returns>
        [DllImport("iomem.dll", EntryPoint = "ReadWithStatus")]
        public static extern bool ReadWithStatus([InAttribute]int printerHandle, [In, OutAttribute]IntPtr resultcde, [InAttribute]int size, [In, OutAttribute]ref int nbytesread, [In, OutAttribute]IntPtr status, [InAttribute]int statusSize, [In, OutAttribute]ref int nbytesreadstatus, [InAttribute]int timeout, [In, OutAttribute]ref int handlearraystopevent, [InAttribute]int nbelements);

        /// <summary>
        /// write data to the printer 
        /// </summary>
        /// <param name="printerHandle">the handle provided by OpenPebble</param>
        /// <param name="cde">data to send to printer</param>
        /// <param name="cdeSize">size of data to send</param>
        /// <returns>the status of the operation</returns>
        [DllImport("iomem.dll", EntryPoint = "WritePebble", CharSet = CharSet.Ansi)]
        public static extern bool WritePebble([InAttribute]int printerHandle, [InAttribute]byte[] cde, [InAttribute]int cdeSize);

        /// <summary>
        /// get the version
        /// </summary>
        /// <param name="version">address of an integer that contains the iomem's version</param>
        /// <returns>the status of the operation</returns>
        [DllImport("iomem.dll", EntryPoint = "GetIomemVersion")]
        public static extern bool GetVersion([OutAttribute]IntPtr version);

        /// <summary>
        /// read the printer status
        /// </summary>
        /// <param name="timeout">in millisecond</param>
        /// <returns>the status of the operation</returns>
        [DllImport("iomem.dll", EntryPoint = "ReadStatusEx")]
        public static extern bool ReadStatusEx([InAttribute]int printerHandle, [In, OutAttribute]IntPtr status, [InAttribute]int answerSize, [In, OutAttribute]ref int dataSize, [InAttribute]int timeout);

        #endregion


        static void Main(string[] args)
        {
            _printerPtr = OpenPebble(_printername);
            if (_printerPtr == 0)
            {
                Console.WriteLine("Error: communication with printer broken");
                Console.WriteLine("Press a key to continue ...");
                Console.ReadKey();
            }
            else
            {
                _status = ReadStatus(30000);
                if (_status !=null)
                {
                    _statusHelper = new EvolisPrinterStatusHelper();

                    Console.WriteLine("Status of your printer: \n");
                    _statusHelper.Update(_status);
                    Console.WriteLine(_statusHelper.Info());
                }
                else
                {
                    Console.WriteLine("Error ReadStatus of your printer: \n");
                }

                ClosePebble(_printerPtr);
                Console.WriteLine("Press a key to continue ...");
                Console.ReadKey();
            }
        }

        /// <summary>
        /// Read the status from the device.
        /// </summary>
        /// <param name="timeout">The timeout.</param>
        /// <returns>device's status</returns>
        /// <exception cref="InternalErrorException">internal software error</exception>
        /// <exception cref="CommunicationErrorException">comm. failed</exception>
        public static byte[] ReadStatus(int timeout)
        {
            IntPtr status = IntPtr.Zero;
            int dataSize = 0;

            try
            {
                status = Marshal.AllocHGlobal(BIGSTRSIZE);
            }
            catch (Exception e)
            {
                // TODO Log e.message
                status = IntPtr.Zero;
            }

            if ((_printerPtr == 0) || (status == IntPtr.Zero))
            {
                return null;
            }

            if (!ReadStatusEx(_printerPtr, status, BIGSTRSIZE, ref dataSize, timeout))
            {
                return null;
            }

            byte[] buffer = null;

            try
            {
                buffer = new byte[dataSize];
                Marshal.Copy(status, buffer, 0, buffer.Length);
            }
            catch (Exception)
            {
                return null;
            }
            finally
            {
                Marshal.FreeHGlobal(status);
            }

            return buffer;
        }
        
    }
}
