/*
    This file is part of the Evolis SDK.

    The Evolis SDK is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis SDK is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once 

#include <stdio.h>
#include <tchar.h>
#include <windows.h>
#include <winspool.h>

#define VERSION "1.9.39.0"

// ------------------------------------------------------------
//	DEFINES
// ------------------------------------------------------------

// EVOLIS_SHORTCUT
// if is defined then a shortcut is defined for the functions Evolis_DriverPrint_*
// for example the shortcut for Evolis_DriverPrint_PrintCard is PrintCard
#define EVOLIS_SHORTCUT

// EVODEBUGLEVEL : 
//		0 - no debug information
//		1 - basic debug information
//		2 - more informations
//		3 - some informations about Card Structure
#define EVODEBUGLEVEL		2

// STDEVO : FILE* Output for debug information
#define STDEVO				stdout

// EVO_DEFAULT_ATTRIB : default attribute for text attribute
#define EVO_DEFAULT_ATTRIB	0


// EVO_STRING_LIMIT : length max. for strings
#define EVO_STRING_LIMIT	1024

// ------------------------------------------------------------
// TYPES
// ------------------------------------------------------------

typedef struct _EVOLIS_CARD
	{
	
		// ---------------- 
	
		HDC		hDC;			// Graphical context
		HANDLE	hPrinter;		// Destination Printer
		
		// ----------------
								// this will be set up with the openCard function
		int		height;			// of the card (see CR80 dimensions)
		int		width;			// idem
		
		// ----------------
		
		BOOL	bNewFontCreated;	// Font management
		HFONT	prevFont;
		HFONT	currFont;
		
		// ----------------
		
		BOOL	bNewPenCreated;		// Pen management
		HPEN	prevPen;
		HPEN	currPen;
				
		// ----------------
		
		DWORD	dwLastError;		// Error Management
		int		bOnError;
		
	} EvolisCard, * PEvolisCard;

// ------------------------------------------------------------
// PROTOTYPES
// ------------------------------------------------------------

//
//===[ START/END DOC COMMANDS ]=================================================================================================================
//
/// @see : http://msdn.microsoft.com/en-us/library/ms535737%28VS.85%29.aspx
//

// ------------------------------------------
//	Evolis_DriverPrint_OpenCard
// ------------------------------------------
/// @brief				Initialize the current printing job
/// @param pCard		the contener for the job
/// @param pPrinterName the printer destination for the job
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_OpenCard			( PEvolisCard pCard, LPTSTR pPrinterName );

// ------------------------------------------
//	Evolis_DriverPrint_PrintCard
// ------------------------------------------
/// @brief				Print the current job and free graphical ressource
/// @param pCard		the contener for the job
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_PrintCard			( PEvolisCard pCard );

//
//===[ BITMAP COMMAND ]=================================================================================================================
//
/// @see : http://msdn.microsoft.com/en-us/library/ms532349(VS.85).aspx
//

// ------------------------------------------
//	Evolis_DriverPrint_AddBitmap
// ------------------------------------------
/// @brief				Add a bitmap to the card. 
///
///						The Bitmap is projected into a rectangle
///
/// @param pCard		the contener for the job
/// @param bmpFile		the full name of the BMP file (with the path)
/// @param nXOffset
/// @param nYOffset		are the coordinates for the upper leftmost point of the rectangle
/// @param nWidth
/// @param nHeight		are the dimensions of the rectangle
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_AddBitmap			( PEvolisCard pCard, LPTSTR bmpFile, int nXOffset, int nYOffset, int nWidth, int nHeight);

//
//===[ TEXT COMMANDS ]=================================================================================================================
//
/// @see : http://msdn.microsoft.com/en-us/library/ms534210(VS.85).aspx
//

// ------------------------------------------
//	Evolis_DriverPrint_SetTextAttrib
// ------------------------------------------
/// @brief				Set the text attributes for the next text command 
///						(Previous text resource are deleted)
/// @param pCard		the contener for the job
/// @param height		the size of the font 
/// @param width		the size of the font (please use EVO_DEFAULT_ATTRIB for automatic computation of this parameter) 
/// @param angle		text orientation (in degree)
/// @param weight		please usee this predefined scale (wingdi.h) for the text weight
//
//							FW_DONTCARE 	0
//							FW_THIN 		100
//							FW_EXTRALIGHT 	200
//							FW_ULTRALIGHT 	200
//							FW_LIGHT 		300
//							FW_NORMAL 		400
//							FW_REGULAR 		400
//							FW_MEDIUM 		500
//							FW_SEMIBOLD 	600
//							FW_DEMIBOLD 	600
//							FW_BOLD 		700
//							FW_EXTRABOLD 	800
//							FW_ULTRABOLD 	800
//							FW_HEAVY 		900
//							FW_BLACK 		900
//
/// @param isUnderline	TRUE to underline the text else FALSE
/// @param isItalic		TRUE to use italic else FALSE
/// @param lpszFace		the name of the font (e.g. "Arial") - please be sure that the font exits (use enumfonts)
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_SetTextAttrib		( PEvolisCard pCard, int height, int width, int angle, int weight, int isUnderline, int isItalic, LPCTSTR lpszFace );

// ------------------------------------------
//	Evolis_DriverPrint_SetColors
// ------------------------------------------
/// @brief				Set the foreground color for the text and the backgroun color
///
/// @param pCard		the contener for the job
/// @param fgColor		the foreground color. Please use the RGB macro define in wingdi.h (e.g. RGB(255,0,0) for Red).
/// @param bgColor		the background color (see below)
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_SetColors			( PEvolisCard pCard, COLORREF fgColor, COLORREF bgColor );

// ------------------------------------------
//	Evolis_DriverPrint_AddText
// ------------------------------------------
/// @brief				Add a text to the card
///	
/// @param pCard		the contener for the job
/// @param strText		the text (no justification)
/// @param nXOffset
/// @param nYOffset		are the coordinates for the upper leftmost point of the band that contains the text
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_AddText				( PEvolisCard pCard, LPTSTR strText, int nTopX, int nTopY );

// ------------------------------------------
//	Evolis_DriverPrint_AddFormatedText
// ------------------------------------------
/// @brief				Add a text to the card
///	
/// @param pCard		the contener for the job
/// @param strText		the text (no justification)
/// @param nFormat		flags to format the text.
///						please use standard flags : DT_BOTTOM, DT_CENTER, DT_LEFT, DT_RIGHT, DT_TOP, DT_VCENTER, ...
///							see http://msdn.microsoft.com/en-us/library/ms533909(VS.85).aspx for further descriptions
/// @param nXOffset
/// @param nYOffset		are the coordinates for the upper leftmost point of the rectangle that contains the text
/// @param nBottomX
///	@param nBottomY		are the coordinates for the opposite point
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------


BOOL Evolis_DriverPrint_AddFormatedText		( PEvolisCard pCard, LPTSTR strText, int nFormat, int nTopX, int nTopY, int nBottomX, int nBottomY  );

// ------------------------------------------
//	Evolis_DriverPrint_MagEncode
// ------------------------------------------
/// @brief				 Encode tracks 1, 2 and 3
///	@note				 the magnetic encoding settings (ISO,HICO/LOCO) must be done via driver properties interface
/// @note				 the datas are transmitted by an escape command (PASSTHROUGH)
/// @param pCard		 the contener for the job
/// @param strDataTrack1 datas for the track 1
/// @param strDataTrack2 datas for the track 2
/// @param strDataTrack3 datas for the track 3
/// @return				 TRUE  if success
///						 FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_MagEncode			( PEvolisCard pCard, LPTSTR strDataTrack1, LPTSTR strDataTrack2, LPTSTR strDataTrack3 );

//
//===[ GEOMETRIC FORMS ]=================================================================================================================
//
/// @see : http://msdn.microsoft.com/en-us/library/ms533275%28VS.85%29.aspx
/// @see : http://msdn.microsoft.com/en-us/library/ms534243(VS.85).aspx
/// @see : http://msdn.microsoft.com/en-us/library/ms536795%28VS.85%29.aspx
//
// WARNING : no ICM support
//

// ------------------------------------------
//	Evolis_DriverPrint_SetLineAttrib
// ------------------------------------------
/// @brief				set the thickness and the color for the line.
///
/// @param pCard		the contener for the job
/// @param nThickness	the thickness of the line
/// @param fgColor		the color (use RGB macro)
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_SetLineAttrib		( PEvolisCard pCard, int nThickness, COLORREF fgColor );

// ------------------------------------------
//	Evolis_DriverPrint_AddLine
// ------------------------------------------
/// @brief				add a line segment to the card
///
/// @param pCard		the contener for the job
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_AddLine				( PEvolisCard pCard, int nStartX, int nStartY, int nEndX, int nEndY );

// ------------------------------------------
//	Evolis_DriverPrint_AddEllipse
// ------------------------------------------
/// @brief				add an ellipse to the card
///
/// @param pCard		the contener for the job
//	
// ( nStartX, nStartY ), (nEndX, nEndY) define a rectangle and the eclipse is inscribed in this rectangle
//
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_AddEllipse			( PEvolisCard pCard, COLORREF fillColor, int nStartX, int nStartY, int nEndX, int nEndY );

// ------------------------------------------
//	Evolis_DriverPrint_AddRectangle
// ------------------------------------------
/// @brief				add a rectangle to the card
///
/// @param pCard		the contener for the job
//	
// ( nStartX, nStartY ) : upper leftmost point, (nEndX, nEndY) define the rectangle 
//
/// @return				TRUE  if success
///						FALSE else
// ------------------------------------------

BOOL Evolis_DriverPrint_AddRectangle		( PEvolisCard pCard, COLORREF fillColor, int nStartX, int nStartY, int nEndX, int nEndY );

#ifdef EVOLIS_SHORTCUT

#define OpenCard		Evolis_DriverPrint_OpenCard
#define PrintCard		Evolis_DriverPrint_PrintCard
#define AddBitmap		Evolis_DriverPrint_AddBitmap
#define SetTextAttrib	Evolis_DriverPrint_SetTextAttrib
#define SetColors		Evolis_DriverPrint_SetColors	
#define AddText			Evolis_DriverPrint_AddText		
#define MagEncode		Evolis_DriverPrint_MagEncode	
#define SetLineAttrib	Evolis_DriverPrint_SetLineAttrib
#define AddLine			Evolis_DriverPrint_AddLine		
#define AddEllipse		Evolis_DriverPrint_AddEllipse	
#define AddRectangle	Evolis_DriverPrint_AddRectangle	
#define AddFormatedText Evolis_DriverPrint_AddFormatedText
#endif 

// -------- Auxiliary functions

// SetBitMapPixels
// 

int		SetBitMapPixels		( HDC *phDC, HBITMAP hBmp, COLORREF rgbTransparent, int nXOffset, int nYOffset, int iW, int iH);

// -- Error Management

DWORD	EvoError			( PEvolisCard pCard, char *errorMsg );
DWORD	EvoInitError		( PEvolisCard pCard );
void	EvoCardPrintInfo	( PEvolisCard pCard );